@@ -438,7 +438,8 @@ static bool import_native_module_sync(JSContext* cx, unsigned argc,
     }
 
     JS::RootedObject native_obj(cx);
-    if (!Gjs::NativeModuleRegistry::get().load(cx, id.get(), &native_obj)) {
+    if (!Gjs::NativeModuleDefineFuncs::get().define(cx, id.get(),
+                                                    &native_obj)) {
         gjs_throw(cx, "Failed to load native module: %s", id.get());
         return false;
     }
@@ -535,23 +536,6 @@ JSObject* gjs_module_resolve(JSContext* cx, JS::HandleValue importingModulePriv,
     return &result.toObject();
 }
 
-// Note: exception is never pending after this function finishes, even if it
-// returns null. The return value is intended to be passed to
-// JS::FinishDynamicModuleImport().
-static JSObject* reject_new_promise_with_pending_exception(JSContext* cx) {
-    JS::ExceptionStack stack{cx};
-    if (!JS::StealPendingExceptionStack(cx, &stack)) {
-        gjs_log_exception(cx);
-        return nullptr;
-    }
-    JS::RootedObject rejected{cx, JS::NewPromiseObject(cx, nullptr)};
-    if (!rejected || !JS::RejectPromise(cx, rejected, stack.exception())) {
-        gjs_log_exception(cx);
-        return nullptr;
-    }
-    return rejected;
-}
-
 // Call JS::FinishDynamicModuleImport() with the values stashed in the function.
 // Can fail in JS::FinishDynamicModuleImport(), but will assert if anything
 // fails in fetching the stashed values, since that would be a serious GJS bug.
@@ -593,12 +577,9 @@ static bool finish_import(JSContext* cx, JS::HandleObject evaluation_promise,
 // case we must not call JS::FinishDynamicModuleImport().
 GJS_JSAPI_RETURN_CONVENTION
 static bool fail_import(JSContext* cx, const JS::CallArgs& args) {
-    if (!JS_IsExceptionPending(cx))
-        return false;
-
-    JS::RootedObject rejected_promise{
-        cx, reject_new_promise_with_pending_exception(cx)};
-    return finish_import(cx, rejected_promise, args);
+    if (JS_IsExceptionPending(cx))
+        return finish_import(cx, nullptr, args);
+    return false;
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -607,16 +588,12 @@ static bool import_rejected(JSContext* cx, unsigned argc, JS::Value* vp) {
 
     gjs_debug(GJS_DEBUG_IMPORTER, "Async import promise rejected");
 
-    // Reject a new promise with the rejection value of the async import
-    // promise, so that FinishDynamicModuleImport will reject the
-    // internal_promise with it.
-    JS::RootedObject rejected{cx, JS::NewPromiseObject(cx, nullptr)};
-    if (!rejected || !JS::RejectPromise(cx, rejected, args.get(0))) {
-        gjs_log_exception(cx);
-        return finish_import(cx, nullptr, args);
-    }
+    // Throw the value that the promise is rejected with, so that
+    // FinishDynamicModuleImport will reject the internal_promise with it.
+    JS_SetPendingException(cx, args.get(0),
+                           JS::ExceptionStackBehavior::DoNotCapture);
 
-    return finish_import(cx, rejected, args);
+    return finish_import(cx, nullptr, args);
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -688,16 +665,9 @@ bool gjs_dynamic_module_resolve(JSContext* cx,
     args[1].setString(specifier);
 
     JS::RootedValue result(cx);
-    if (!JS::Call(cx, loader, "moduleResolveAsyncHook", args, &result)) {
-        if (!JS_IsExceptionPending(cx))
-            return false;
-
-        JS::RootedObject rejected_promise{
-            cx, reject_new_promise_with_pending_exception(cx)};
-        return JS::FinishDynamicModuleImport(cx, rejected_promise,
-                                             importing_module_priv,
+    if (!JS::Call(cx, loader, "moduleResolveAsyncHook", args, &result))
+        return JS::FinishDynamicModuleImport(cx, nullptr, importing_module_priv,
                                              module_request, internal_promise);
-    }
 
     // Release in finish_import
     GjsContextPrivate* priv = GjsContextPrivate::from_cx(cx);
